using System;
using NUnit.Extensions.Converter.Evaluators;

namespace NUnit.Extensions.Converter
{
	abstract public class EvaluationConverter : SpecificObjectConverter
	{
		private CompositeEvaluator highEvaluators = new CompositeEvaluator();
		private CompositeEvaluator mediumEvaluators = new CompositeEvaluator();
		private CompositeEvaluator lowEvaluators = new CompositeEvaluator();
		private CompositeEvaluator evaluators = new CompositeEvaluator();
		private object[] evaluationKeys = {};
		public EvaluationConverter(Converter converter) : base(converter)
		{
			evaluators.AddEvaluator(highEvaluators);
			evaluators.AddEvaluator(mediumEvaluators);
			evaluators.AddEvaluator(lowEvaluators);
		}
		public override void AddEvaluator(Evaluator evaluator)
		{
			switch (evaluator.ThisPriority) {
			case Evaluator.Priority.HIGH:
				highEvaluators.AddEvaluator(evaluator);
				break;
			case Evaluator.Priority.MEDIUM:
				mediumEvaluators.AddEvaluator(evaluator);
				break;
			case Evaluator.Priority.LOW:
				lowEvaluators.AddEvaluator(evaluator);
				break;
			}
		}
		public void SetEvaluationKeys(object[] evaluationKeys)
		{
			this.evaluationKeys = evaluationKeys;
		}
		protected override object ConvertSelf(object o)
		{
			evaluators.SetConverter(outermostConverter);
			object[] result = new object[evaluationKeys.Length];
			for (int i = 0; i < result.Length; ++i) {
				result[i] = evaluators.Evaluate(o, evaluationKeys[i]);
			}
			return result.Length == 1 ? result[0] : result;
		}
	}
}
