using System;
using System.Collections;

namespace NUnit.Extensions.Converter.Evaluators
{
	public class CompositeEvaluator : Evaluator
	{
		private IList evaluators = new ArrayList();
		public void AddEvaluator(Evaluator evaluator)
		{
			evaluators.Add(evaluator);
		}
		public override object Evaluate(object o, object key)
		{
			if (Converter.NotVisible(o))
				return o;
			object result = o;
			foreach (Evaluator evaluator in evaluators) {
				result = evaluator.Evaluate(result, key);
				if (Converter.NotVisible(result))
					return result;
			}
			return result;
		}
		public override void SetConverter(Converter converter)
		{
			foreach (Evaluator evaluator in evaluators) {
				evaluator.SetConverter(converter);
			}
			base.SetConverter(converter);
		}
	}
}
