using System;
using System.Reflection;

namespace NUnit.Extensions.Converter
{
	public class IndexerEvaluator : Evaluator
	{
		public override object Evaluate(object o, object key)
		{
			Type type = o.GetType();
			object[] paramsValue;
			Type[] paramsType;

			paramsValue = key as object[];
			if (paramsValue != null)
			{
				paramsType = new Type[paramsValue.Length];
				for(int i = 0; i < paramsType.Length; ++i)
				{
					paramsType[i] = paramsValue[i].GetType();
				}
			}
			else 
			{
				paramsValue = new object[] { key };
				paramsType = new Type[] { key.GetType() };
			}
			PropertyInfo info = type.GetProperty("Item", paramsType);
			return info.GetValue(o, paramsValue);
		}
	}
	public class IndexerValueConverter : EvaluationConverter
	{
		private string format;
		private Type type;
		private object[] keys;

		public IndexerValueConverter(Converter converter, Type type, params object[] keys)
			: this(converter, null, type, keys)
		{
		}
		public IndexerValueConverter(Converter converter, string format, Type type, params object[] keys)
			: base(converter)
		{
			this.format = format;
			this.type = type;
			this.keys = keys;
			AddEvaluator(new IndexerEvaluator());
			if (keys.Length == 0)
				throw new ArgumentException("Indexer keys is empty.");
		}
		protected override bool CanConvertSelf(object o)
		{
			return type.IsInstanceOfType(o);
		}
		protected override object ConvertSelf(object o)
		{
			SetEvaluationKeys(keys);
			object result = base.ConvertSelf(o);
			if (format != null) 
			{
				Formatter formatter = new Formatter(outermostConverter);
				return formatter.Format(format, result);
			}
			return result;
		}
	}
}
